@@ -1,4 +1,4 @@
-/* Copyright (C) 2014 and 2015 Chris Vine
+/* Copyright (C) 2014, 2016 and 2020 Chris Vine
 
 The library comprised in this file or of which this file is part is
 distributed by Chris Vine under the GNU Lesser General Public
@@ -75,8 +75,8 @@ later.
  *      - with Extension::exec() and Extension::exec_shared(), it is
  *        trivial to do (see the example below)
  *
- * To call Extension::exec() or Extension::exec_shared(), guile-2.0 is
- * required.
+ * To call Extension::exec() or Extension::exec_shared(), guile-2.0 >=
+ * 2.0.2, guile-2.2 >= 2.1.3 or guile-3.0 >= 3.0.0 is required.
  *
  * Usage
  * -----
@@ -319,8 +319,6 @@ later.
  * task in order to make them visible to the task, but that would be
  * done safely if they have previously been loaded in another task.)
  *
- * This issue is likely to be fixed in a future version of guile.
- *
  * If a C++ program is to run guile tasks on a TaskManager object
  * having a maximum thread count greater than one (or in more than one
  * thread in some other way), one other point should be noted.  When a
@@ -361,16 +359,42 @@ later.
  * or Extension::exec_shared(), the program which calls it will need
  * to link itself explicitly with libguile as well as c++-gtk-utils,
  * and to do that will need to use pkg-config to obtain libguile's
- * cflags and libs particulars (its pkg-config file is guile-2.0.pc).
- * This library does NOT do that for you.  Secondly, by linking with
- * libguile you will be governed by the LGPL version 3 or later,
- * instead of the LGPL version 2.1 or later, with respect to that
- * linking.  That's fine (there is nothing wrong with the LGPL version
- * 3 and this library permits that) but you should be aware of it.
- * The scheme code in guile's scheme level modules is also in the main
- * released under the LGPL version 3 or later ("in the main" because
- * readline.scm, comprised in the readline module, is released under
- * the GPL version 3 or later).
+ * cflags and libs particulars (its pkg-config file is guile-2.0.pc,
+ * guile-2.2.pc or guile-3.0.pc).  This library does NOT do that for
+ * you.  Secondly, by linking with libguile you will be governed by
+ * the LGPL version 3 or later, instead of the LGPL version 2.1 or
+ * later, with respect to that linking.  That's fine (there is nothing
+ * wrong with the LGPL version 3 and this library permits that) but
+ * you should be aware of it.  The scheme code in guile's scheme level
+ * modules is also in the main released under the LGPL version 3 or
+ * later ("in the main" because readline.scm, comprised in the
+ * readline module, is released under the GPL version 3 or later).
+ *
+ * Configuration
+ * -------------
+ *
+ * By default, when the c++-gtk-utils library is configured,
+ * configuration will first look for guile-3.0 >= 3.0.0, then if it
+ * does not find that it will look for guile-2.2 >= 2.1.3, then if it
+ * does not find that it will look for guile-2.0 >= 2.0.2, and then if
+ * it finds none it will disable guile support; and the library header
+ * files will then be set up appropriately.  guile-3.0, guile-2.2 or
+ * guile-2.0 can be specifically picked with the \--with-guile=3.0,
+ * \--with-guile=2.2 or \--with-guile=2.0 configuration options
+ * respectively.  Guile support can be omitted with the
+ * \--with-guile=no option.
+
+ * However, as mentioned under "Licence" above, any program using
+ * Extension::exec() or Extension::exec_shared() must link itself
+ * explicitly with libguile via either guile-2.0.pc (for guile-2.0),
+ * guile-2.2.pc (for guile-2.2) or guile-3.0.pc (for guile-3.0).
+ * Programs should use whichever of those is the one for which
+ * c++-gtk-utils was configured.  If you get link-time messages from a
+ * program about being unable to link to scm_dynwind_block_asyncs(),
+ * then there has been a version mismatch.  If you get link-time
+ * messages about being unable to link to Cgu::Extension::init_mutex()
+ * or Cgu::Extension::get_user_module_mutex() then c++-gtk-utils has
+ * not been configured to offer guile support.
  *
  * Custom translators
  * ------------------
@@ -416,6 +440,17 @@ later.
  * exhaustion in such a case probably spells doom for the program
  * concerned anyway, if it has other work to do.
  *
+ * Note also that code in a custom translator should not store guile
+ * SCM objects (which are pointers to guile scheme objects) in memory
+ * blocks allocated by malloc() or the new expression, or they will
+ * not be seen by the garbage collector used by libguile (which is the
+ * gc library) and therefore may be prematurely deallocated.  To keep
+ * such items alive in custom translators, SCM variables should be
+ * kept as local variables or parameter names in functions (and so
+ * stored on the stack or in registers, where they will be seen by the
+ * garbage collector), or in memory allocated with scm_gc_malloc(),
+ * where they will also be seen by the garbage collector.
+ *
  * Scheme
  * ------
  * If you want to learn more about scheme, these are useful sources:
@@ -591,7 +626,18 @@ void guile_wrapper_cb2(TransType* translator,
     scm_dynwind_begin(scm_t_dynwind_flags(0));
     scm_dynwind_unwind_handler(&cgu_unlock_module_mutex, 0, SCM_F_WIND_EXPLICITLY);
     get_user_module_mutex()->lock(); // won't throw
+    // neither make-fresh-user-module nor set-module-declarative?! are
+    // part of the publicly documented guile API, but are made
+    // available by ice-9/boot-9.scm.  set-module-declarative?! is the
+    // modifier of the declarative? field of the module record type in
+    // guile-3.0.  There is no workable way of avoiding their use, but
+    // they should be safe until at least the next API-breaking
+    // version of guile, and probably long after that.
     SCM new_mod = scm_call_0(scm_c_public_ref("guile", "make-fresh-user-module"));
+#if SCM_MAJOR_VERSION >= 3
+    scm_call_2(scm_c_public_ref("guile", "set-module-declarative?!"),
+	       new_mod, SCM_BOOL_F);
+#endif
     scm_dynwind_end();
 
     scm = scm_eval_string_in_module(scm_from_utf8_string(loader->c_str()),
