@@ -233,6 +233,7 @@ struct FutureWhenError: public std::exception {
  * the return type for the 'when' callback indicating no result.
  */
 
+#ifndef DOXYGEN_PARSING
 namespace FutureHelper {
 
 // the sole purpose of this struct is to enable a callback object to
@@ -268,6 +269,7 @@ struct WhenWrapperArgRel {
 };
 
 } // namespace FutureHelper
+#endif // DOXYGEN_PARSING
 
 
 template <class Val>
@@ -859,13 +861,15 @@ public:
  * The error flag will also have been set if the worker thread is
  * cancelled or the thread wrapper in this Cgu::Thread::Future object
  * threw std::bad_alloc.  On the error flag being set, this method
- * will unblock.  This method is thread safe and may be called by any
- * thread (and by more than one thread).  It is a cancellation point
- * if it blocks, and from version 2.0.11 is cancellation safe if the
- * stack unwinds on cancellation.  It is also strongly exception safe:
- * no data will be lost if extracting the value fails.
+ * will unblock and return a default constructed object of the return
+ * type.  This method is thread safe and may be called by any thread
+ * (and by more than one thread).  It is a cancellation point if it
+ * blocks, and from version 2.0.11 is cancellation safe if the stack
+ * unwinds on cancellation.  It is also strongly exception safe: no
+ * data will be lost if extracting the value fails.
  * @return The value obtained from the function which is represented
- * by this object
+ * by this object, or a default constructed object of the return type
+ * if the error flag has been set.
  * @exception Cgu::Thread::FutureThreadError This method might throw
  * Cgu::Thread::FutureThreadError if run() has not previously been
  * called and the thread did not start properly when this function
@@ -933,12 +937,14 @@ public:
  * The error flag will also have been set if the worker thread is
  * cancelled or the thread wrapper in this Cgu::Thread::Future object
  * threw std::bad_alloc.  On the error flag being set, this method
- * will unblock.  This method is a cancellation point if it blocks,
- * and is cancellation safe if the stack unwinds on cancellation.
- * This method is only exception safe if the return type's move
- * constructor is exception safe.
+ * will unblock and return a default constructed object of the return
+ * type.  This method is a cancellation point if it blocks, and is
+ * cancellation safe if the stack unwinds on cancellation.  This
+ * method is only exception safe if the return type's move constructor
+ * is exception safe.
  * @return The value obtained from the function which is represented
- * by this object
+ * by this object, or a default constructed object of the return type
+ * if the error flag has been set.
  * @exception Cgu::Thread::FutureThreadError This method might throw
  * Cgu::Thread::FutureThreadError if run() has not previously been
  * called and the thread did not start properly when this function
