@@ -1079,6 +1079,12 @@ public:
   CGU_GLIB_MEMORY_SLICES_FUNCS
 };
 
+// we don't want glib's warnings about GStaticRecMutex
+#ifdef CGU_USE_DIAGNOSTIC_PRAGMAS
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
+
 /**
  * @class GrecmutexLock mutex.h c++-gtk-utils/mutex.h
  * @brief A scoped locking class for exception safe locking of
@@ -1154,6 +1160,10 @@ public:
   CGU_GLIB_MEMORY_SLICES_FUNCS
 };
 
+#ifdef CGU_USE_DIAGNOSTIC_PRAGMAS
+#pragma GCC diagnostic pop
+#endif
+
 } // namespace Thread
 
 } // namespace Cgu
