@@ -1,4 +1,4 @@
-/* Copyright (C) 2012 and 2013 Chris Vine
+/* Copyright (C) 2012, 2013 and 2016 Chris Vine
 
 The library comprised in this file or of which this file is part is
 distributed by Chris Vine under the GNU Lesser General Public
@@ -26,6 +26,8 @@ License as follows:
 
 #include <algorithm>  // for std::min()
 #include <cstdlib>    // for std::abs()
+#include <limits>     // for std::numeric_limits
+#include <stdexcept>  // for std::length_error
 
 #include <glib.h>     // for g_critical()
 
@@ -261,6 +263,12 @@ unsigned int TaskManager::get_tasks() const {
   return ref_impl->tasks;
 }
 
+unsigned int TaskManager::init_max_tasks() {
+  std::deque<QueueItemType> d;
+  return (d.max_size() < std::numeric_limits<unsigned int>::max()) ?
+         d.max_size() : std::numeric_limits<unsigned int>::max();
+}
+
 // this method should only be entered when holding ref_impl's mutex
 void TaskManager::set_max_threads_impl(unsigned int max, Mutex::TrackLock& lock) {
 
@@ -386,9 +394,11 @@ void TaskManager::stop_all() {
 
     // we could be adding more KillThread callbacks than necessary
     // here, because as we are doing this a timeout on a thread might
-    // expire leading to its demise in that way.  However, that
-    // doesn't matter - we just get left with a redundant callback in
-    // 'task_queue' which never gets used
+    // expire leading to its demise in that way, or a concurrent call
+    // to change_max_threads() or add_task() may have failed to start
+    // a thread and thrown an exception.  However, that doesn't matter
+    // - we just get left with a redundant callback in 'task_queue'
+    // which never gets used
     for (unsigned int count = ref_impl->used_threads; count; --count) {
       try {
 	ref_impl->task_queue.emplace(
@@ -436,6 +446,9 @@ void TaskManager::add_task(
   { // scope block for mutex lock
     Mutex::TrackLock lock{ref_impl->mutex};
 
+    if (ref_impl->tasks >= get_max_tasks())
+      throw std::length_error("TaskManager::add_task: maximum size of TaskManager exceeded");
+
     if (ref_impl->error || ref_impl->stop_status != StopStatus::running)
       throw TaskError();
 
